@@ -7,22 +7,31 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdbool.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/param.h>
 #include <sys/types.h>
 
+#ifndef MIN
+#define MIN(a,b) ( (a) > (b) ? (b) : (a) )
+#endif
+#ifndef MAX
+#define MAX(a,b) ( (a) > (b) ? (a) : (b) )
+#endif
+
+#ifdef WINDOWS
+typedef unsigned long long u_int64_t;
+#endif
 
 /* Structure to save state of computation between the single steps.  */
 struct sha512_ctx
 {
-  uint64_t H[8];
+  u_int64_t H[8];
 
-  uint64_t total[2];
-  uint64_t buflen;
-  char buffer[256];	/* NB: always correctly aligned for uint64_t.  */
+  u_int64_t total[2];
+  u_int64_t buflen;
+  char buffer[256];	/* NB: always correctly aligned for u_int64_t.  */
 };
 
 
@@ -47,48 +56,48 @@ static const unsigned char fillbuf[128] = { 0x80, 0 /* , 0, 0, ...  */ };
 
 
 /* Constants for SHA512 from FIPS 180-2:4.2.3.  */
-static const uint64_t K[80] =
+static const u_int64_t K[80] =
   {
-    UINT64_C (0x428a2f98d728ae22), UINT64_C (0x7137449123ef65cd),
-    UINT64_C (0xb5c0fbcfec4d3b2f), UINT64_C (0xe9b5dba58189dbbc),
-    UINT64_C (0x3956c25bf348b538), UINT64_C (0x59f111f1b605d019),
-    UINT64_C (0x923f82a4af194f9b), UINT64_C (0xab1c5ed5da6d8118),
-    UINT64_C (0xd807aa98a3030242), UINT64_C (0x12835b0145706fbe),
-    UINT64_C (0x243185be4ee4b28c), UINT64_C (0x550c7dc3d5ffb4e2),
-    UINT64_C (0x72be5d74f27b896f), UINT64_C (0x80deb1fe3b1696b1),
-    UINT64_C (0x9bdc06a725c71235), UINT64_C (0xc19bf174cf692694),
-    UINT64_C (0xe49b69c19ef14ad2), UINT64_C (0xefbe4786384f25e3),
-    UINT64_C (0x0fc19dc68b8cd5b5), UINT64_C (0x240ca1cc77ac9c65),
-    UINT64_C (0x2de92c6f592b0275), UINT64_C (0x4a7484aa6ea6e483),
-    UINT64_C (0x5cb0a9dcbd41fbd4), UINT64_C (0x76f988da831153b5),
-    UINT64_C (0x983e5152ee66dfab), UINT64_C (0xa831c66d2db43210),
-    UINT64_C (0xb00327c898fb213f), UINT64_C (0xbf597fc7beef0ee4),
-    UINT64_C (0xc6e00bf33da88fc2), UINT64_C (0xd5a79147930aa725),
-    UINT64_C (0x06ca6351e003826f), UINT64_C (0x142929670a0e6e70),
-    UINT64_C (0x27b70a8546d22ffc), UINT64_C (0x2e1b21385c26c926),
-    UINT64_C (0x4d2c6dfc5ac42aed), UINT64_C (0x53380d139d95b3df),
-    UINT64_C (0x650a73548baf63de), UINT64_C (0x766a0abb3c77b2a8),
-    UINT64_C (0x81c2c92e47edaee6), UINT64_C (0x92722c851482353b),
-    UINT64_C (0xa2bfe8a14cf10364), UINT64_C (0xa81a664bbc423001),
-    UINT64_C (0xc24b8b70d0f89791), UINT64_C (0xc76c51a30654be30),
-    UINT64_C (0xd192e819d6ef5218), UINT64_C (0xd69906245565a910),
-    UINT64_C (0xf40e35855771202a), UINT64_C (0x106aa07032bbd1b8),
-    UINT64_C (0x19a4c116b8d2d0c8), UINT64_C (0x1e376c085141ab53),
-    UINT64_C (0x2748774cdf8eeb99), UINT64_C (0x34b0bcb5e19b48a8),
-    UINT64_C (0x391c0cb3c5c95a63), UINT64_C (0x4ed8aa4ae3418acb),
-    UINT64_C (0x5b9cca4f7763e373), UINT64_C (0x682e6ff3d6b2b8a3),
-    UINT64_C (0x748f82ee5defb2fc), UINT64_C (0x78a5636f43172f60),
-    UINT64_C (0x84c87814a1f0ab72), UINT64_C (0x8cc702081a6439ec),
-    UINT64_C (0x90befffa23631e28), UINT64_C (0xa4506cebde82bde9),
-    UINT64_C (0xbef9a3f7b2c67915), UINT64_C (0xc67178f2e372532b),
-    UINT64_C (0xca273eceea26619c), UINT64_C (0xd186b8c721c0c207),
-    UINT64_C (0xeada7dd6cde0eb1e), UINT64_C (0xf57d4f7fee6ed178),
-    UINT64_C (0x06f067aa72176fba), UINT64_C (0x0a637dc5a2c898a6),
-    UINT64_C (0x113f9804bef90dae), UINT64_C (0x1b710b35131c471b),
-    UINT64_C (0x28db77f523047d84), UINT64_C (0x32caab7b40c72493),
-    UINT64_C (0x3c9ebe0a15c9bebc), UINT64_C (0x431d67c49c100d4c),
-    UINT64_C (0x4cc5d4becb3e42b6), UINT64_C (0x597f299cfc657e2a),
-    UINT64_C (0x5fcb6fab3ad6faec), UINT64_C (0x6c44198c4a475817)
+    0x428a2f98d728ae22ULL, 0x7137449123ef65cdULL,
+    0xb5c0fbcfec4d3b2fULL, 0xe9b5dba58189dbbcULL,
+    0x3956c25bf348b538ULL, 0x59f111f1b605d019ULL,
+    0x923f82a4af194f9bULL, 0xab1c5ed5da6d8118ULL,
+    0xd807aa98a3030242ULL, 0x12835b0145706fbeULL,
+    0x243185be4ee4b28cULL, 0x550c7dc3d5ffb4e2ULL,
+    0x72be5d74f27b896fULL, 0x80deb1fe3b1696b1ULL,
+    0x9bdc06a725c71235ULL, 0xc19bf174cf692694ULL,
+    0xe49b69c19ef14ad2ULL, 0xefbe4786384f25e3ULL,
+    0x0fc19dc68b8cd5b5ULL, 0x240ca1cc77ac9c65ULL,
+    0x2de92c6f592b0275ULL, 0x4a7484aa6ea6e483ULL,
+    0x5cb0a9dcbd41fbd4ULL, 0x76f988da831153b5ULL,
+    0x983e5152ee66dfabULL, 0xa831c66d2db43210ULL,
+    0xb00327c898fb213fULL, 0xbf597fc7beef0ee4ULL,
+    0xc6e00bf33da88fc2ULL, 0xd5a79147930aa725ULL,
+    0x06ca6351e003826fULL, 0x142929670a0e6e70ULL,
+    0x27b70a8546d22ffcULL, 0x2e1b21385c26c926ULL,
+    0x4d2c6dfc5ac42aedULL, 0x53380d139d95b3dfULL,
+    0x650a73548baf63deULL, 0x766a0abb3c77b2a8ULL,
+    0x81c2c92e47edaee6ULL, 0x92722c851482353bULL,
+    0xa2bfe8a14cf10364ULL, 0xa81a664bbc423001ULL,
+    0xc24b8b70d0f89791ULL, 0xc76c51a30654be30ULL,
+    0xd192e819d6ef5218ULL, 0xd69906245565a910ULL,
+    0xf40e35855771202aULL, 0x106aa07032bbd1b8ULL,
+    0x19a4c116b8d2d0c8ULL, 0x1e376c085141ab53ULL,
+    0x2748774cdf8eeb99ULL, 0x34b0bcb5e19b48a8ULL,
+    0x391c0cb3c5c95a63ULL, 0x4ed8aa4ae3418acbULL,
+    0x5b9cca4f7763e373ULL, 0x682e6ff3d6b2b8a3ULL,
+    0x748f82ee5defb2fcULL, 0x78a5636f43172f60ULL,
+    0x84c87814a1f0ab72ULL, 0x8cc702081a6439ecULL,
+    0x90befffa23631e28ULL, 0xa4506cebde82bde9ULL,
+    0xbef9a3f7b2c67915ULL, 0xc67178f2e372532bULL,
+    0xca273eceea26619cULL, 0xd186b8c721c0c207ULL,
+    0xeada7dd6cde0eb1eULL, 0xf57d4f7fee6ed178ULL,
+    0x06f067aa72176fbaULL, 0x0a637dc5a2c898a6ULL,
+    0x113f9804bef90daeULL, 0x1b710b35131c471bULL,
+    0x28db77f523047d84ULL, 0x32caab7b40c72493ULL,
+    0x3c9ebe0a15c9bebcULL, 0x431d67c49c100d4cULL,
+    0x4cc5d4becb3e42b6ULL, 0x597f299cfc657e2aULL,
+    0x5fcb6fab3ad6faecULL, 0x6c44198c4a475817ULL
   };
 
 
@@ -97,16 +106,16 @@ static const uint64_t K[80] =
 static void
 sha512_process_block (const void *buffer, size_t len, struct sha512_ctx *ctx)
 {
-  const uint64_t *words = buffer;
-  size_t nwords = len / sizeof (uint64_t);
-  uint64_t a = ctx->H[0];
-  uint64_t b = ctx->H[1];
-  uint64_t c = ctx->H[2];
-  uint64_t d = ctx->H[3];
-  uint64_t e = ctx->H[4];
-  uint64_t f = ctx->H[5];
-  uint64_t g = ctx->H[6];
-  uint64_t h = ctx->H[7];
+  const u_int64_t *words = buffer;
+  size_t nwords = len / sizeof (u_int64_t);
+  u_int64_t a = ctx->H[0];
+  u_int64_t b = ctx->H[1];
+  u_int64_t c = ctx->H[2];
+  u_int64_t d = ctx->H[3];
+  u_int64_t e = ctx->H[4];
+  u_int64_t f = ctx->H[5];
+  u_int64_t g = ctx->H[6];
+  u_int64_t h = ctx->H[7];
 
   /* First increment the byte count.  FIPS 180-2 specifies the possible
      length of the file up to 2^128 bits.  Here we only compute the
@@ -119,15 +128,15 @@ sha512_process_block (const void *buffer, size_t len, struct sha512_ctx *ctx)
      the loop.  */
   while (nwords > 0)
     {
-      uint64_t W[80];
-      uint64_t a_save = a;
-      uint64_t b_save = b;
-      uint64_t c_save = c;
-      uint64_t d_save = d;
-      uint64_t e_save = e;
-      uint64_t f_save = f;
-      uint64_t g_save = g;
-      uint64_t h_save = h;
+      u_int64_t W[80];
+      u_int64_t a_save = a;
+      u_int64_t b_save = b;
+      u_int64_t c_save = c;
+      u_int64_t d_save = d;
+      u_int64_t e_save = e;
+      u_int64_t f_save = f;
+      u_int64_t g_save = g;
+      u_int64_t h_save = h;
       unsigned int t;
       /* Operators defined in FIPS 180-2:4.1.2.  */
 #define Ch(x, y, z) ((x & y) ^ (~x & z))
@@ -153,8 +162,8 @@ sha512_process_block (const void *buffer, size_t len, struct sha512_ctx *ctx)
       /* The actual computation according to FIPS 180-2:6.3.2 step 3.  */
       for (t = 0; t < 80; ++t)
 	{
-	  uint64_t T1 = h + S1 (e) + Ch (e, f, g) + K[t] + W[t];
-	  uint64_t T2 = S0 (a) + Maj (a, b, c);
+	  u_int64_t T1 = h + S1 (e) + Ch (e, f, g) + K[t] + W[t];
+	  u_int64_t T2 = S0 (a) + Maj (a, b, c);
 	  h = g;
 	  g = f;
 	  f = e;
@@ -197,14 +206,14 @@ sha512_process_block (const void *buffer, size_t len, struct sha512_ctx *ctx)
 static void
 sha512_init_ctx (struct sha512_ctx *ctx)
 {
-  ctx->H[0] = UINT64_C (0x6a09e667f3bcc908);
-  ctx->H[1] = UINT64_C (0xbb67ae8584caa73b);
-  ctx->H[2] = UINT64_C (0x3c6ef372fe94f82b);
-  ctx->H[3] = UINT64_C (0xa54ff53a5f1d36f1);
-  ctx->H[4] = UINT64_C (0x510e527fade682d1);
-  ctx->H[5] = UINT64_C (0x9b05688c2b3e6c1f);
-  ctx->H[6] = UINT64_C (0x1f83d9abfb41bd6b);
-  ctx->H[7] = UINT64_C (0x5be0cd19137e2179);
+  ctx->H[0] = 0x6a09e667f3bcc908ULL;
+  ctx->H[1] = 0xbb67ae8584caa73bULL;
+  ctx->H[2] = 0x3c6ef372fe94f82bULL;
+  ctx->H[3] = 0xa54ff53a5f1d36f1ULL;
+  ctx->H[4] = 0x510e527fade682d1ULL;
+  ctx->H[5] = 0x9b05688c2b3e6c1fULL;
+  ctx->H[6] = 0x1f83d9abfb41bd6bULL;
+  ctx->H[7] = 0x5be0cd19137e2179ULL;
 
   ctx->total[0] = ctx->total[1] = 0;
   ctx->buflen = 0;
@@ -220,7 +229,7 @@ static void *
 sha512_finish_ctx (struct sha512_ctx *ctx, void *resbuf)
 {
   /* Take yet unprocessed bytes into account.  */
-  uint64_t bytes = ctx->buflen;
+  u_int64_t bytes = ctx->buflen;
   size_t pad;
   unsigned int i;
 
@@ -233,8 +242,8 @@ sha512_finish_ctx (struct sha512_ctx *ctx, void *resbuf)
   memcpy (&ctx->buffer[bytes], fillbuf, pad);
 
   /* Put the 128-bit file length in *bits* at the end of the buffer.  */
-  *(uint64_t *) &ctx->buffer[bytes + pad + 8] = SWAP (ctx->total[0] << 3);
-  *(uint64_t *) &ctx->buffer[bytes + pad] = SWAP ((ctx->total[1] << 3) |
+  *(u_int64_t *) &ctx->buffer[bytes + pad + 8] = SWAP (ctx->total[0] << 3);
+  *(u_int64_t *) &ctx->buffer[bytes + pad] = SWAP ((ctx->total[1] << 3) |
 						  (ctx->total[0] >> 61));
 
   /* Process last bytes.  */
@@ -242,7 +251,7 @@ sha512_finish_ctx (struct sha512_ctx *ctx, void *resbuf)
 
   /* Put result from CTX in first 64 bytes following RESBUF.  */
   for (i = 0; i < 8; ++i)
-    ((uint64_t *) resbuf)[i] = SWAP (ctx->H[i]);
+    ((u_int64_t *) resbuf)[i] = SWAP (ctx->H[i]);
 
   return resbuf;
 }
@@ -282,9 +291,9 @@ sha512_process_bytes (const void *buffer, size_t len, struct sha512_ctx *ctx)
 /* To check alignment gcc has an appropriate operator.  Other
    compilers don't.  */
 # if __GNUC__ >= 2
-#  define UNALIGNED_P(p) (((uintptr_t) p) % __alignof__ (uint64_t) != 0)
+#  define UNALIGNED_P(p) (((uintptr_t) p) % __alignof__ (u_int64_t) != 0)
 # else
-#  define UNALIGNED_P(p) (((uintptr_t) p) % sizeof (uint64_t) != 0)
+#  define UNALIGNED_P(p) (((uintptr_t) p) % sizeof (u_int64_t) != 0)
 # endif
       if (UNALIGNED_P (buffer))
 	while (len > 128)
@@ -346,9 +355,9 @@ static char *
 sha512_crypt_r (const char *key, const char *salt, char *buffer, int buflen)
 {
   unsigned char alt_result[64]
-    __attribute__ ((__aligned__ (__alignof__ (uint64_t))));
+    __attribute__ ((__aligned__ (__alignof__ (u_int64_t))));
   unsigned char temp_result[64]
-    __attribute__ ((__aligned__ (__alignof__ (uint64_t))));
+    __attribute__ ((__aligned__ (__alignof__ (u_int64_t))));
   struct sha512_ctx ctx;
   struct sha512_ctx alt_ctx;
   size_t salt_len;
@@ -386,21 +395,21 @@ sha512_crypt_r (const char *key, const char *salt, char *buffer, int buflen)
   salt_len = MIN (strcspn (salt, "$"), SALT_LEN_MAX);
   key_len = strlen (key);
 
-  if ((key - (char *) 0) % __alignof__ (uint64_t) != 0)
+  if ((key - (char *) 0) % __alignof__ (u_int64_t) != 0)
     {
-      char *tmp = (char *) alloca (key_len + __alignof__ (uint64_t));
+      char *tmp = (char *) alloca (key_len + __alignof__ (u_int64_t));
       key = copied_key =
-	memcpy (tmp + __alignof__ (uint64_t)
-		- (tmp - (char *) 0) % __alignof__ (uint64_t),
+	memcpy (tmp + __alignof__ (u_int64_t)
+		- (tmp - (char *) 0) % __alignof__ (u_int64_t),
 		key, key_len);
     }
 
-  if ((salt - (char *) 0) % __alignof__ (uint64_t) != 0)
+  if ((salt - (char *) 0) % __alignof__ (u_int64_t) != 0)
     {
-      char *tmp = (char *) alloca (salt_len + __alignof__ (uint64_t));
+      char *tmp = (char *) alloca (salt_len + __alignof__ (u_int64_t));
       salt = copied_salt =
-	memcpy (tmp + __alignof__ (uint64_t)
-		- (tmp - (char *) 0) % __alignof__ (uint64_t),
+	memcpy (tmp + __alignof__ (u_int64_t)
+		- (tmp - (char *) 0) % __alignof__ (u_int64_t),
 		salt, salt_len);
     }
 
@@ -518,18 +527,20 @@ sha512_crypt_r (const char *key, const char *salt, char *buffer, int buflen)
 
   /* Now we can construct the result string.  It consists of three
      parts.  */
-  cp = stpncpy (buffer, sha512_salt_prefix, MAX (0, buflen));
+  cp = strncpy (buffer, sha512_salt_prefix, MAX (0, buflen));
+  cp += MIN(strlen(sha512_salt_prefix), MAX (0, buflen));
   buflen -= sizeof (sha512_salt_prefix) - 1;
 
   if (rounds_custom)
     {
-      int n = snprintf (cp, MAX (0, buflen), "%s%zu$",
-			sha512_rounds_prefix, rounds);
+      int n = snprintf (cp, MAX (0, buflen), "%s%u$",
+			sha512_rounds_prefix, (unsigned int)rounds);
       cp += n;
       buflen -= n;
     }
 
-  cp = stpncpy (cp, salt, MIN ((size_t) MAX (0, buflen), salt_len));
+  cp = strncpy (cp, salt, MIN ((size_t) MAX (0, buflen), salt_len));
+  cp += MIN(strlen(salt), MIN ((size_t) MAX (0, buflen), salt_len));
   buflen -= MIN ((size_t) MAX (0, buflen), salt_len);
 
   if (buflen > 0)
